<?php
if ( ! defined( 'ABSPATH' ) ) exit; // Exit if accessed directly

/**
 * Plugin Name: WooCommerce Cryptocoin Payments Gateway
 * Plugin URI: https://www.bropay.io/
 * Description:  Provides a Cryptocoin Payments Gateway.
 * Author: bropay.io
 * Author URI: https://www.bropay.io/
 * Version: 1.0.14
 */

/**
 * Bropay.io Gateway
 * Based on the PayPal Standard Payment Gateway
 *
 * Provides a Cryptocoin Payments Gateway.
 *
 * @class 		WC_Bropay
 * @extends		WC_Gateway_Bropay
 * @version		1.0.14
 * @package		WooCommerce/Classes/Payment
 * @author 		Bropay.io based on PayPal module by WooThemes
 */

add_action( 'plugins_loaded', 'bropay_gateway_load', 0 );
function bropay_gateway_load() {

    if ( ! class_exists( 'WC_Payment_Gateway' ) ) {
        // oops!
        return;
    }

    /**
     * Add the gateway to WooCommerce.
     */
    add_filter( 'woocommerce_payment_gateways', 'wc_bropay_add_gateway' );

    function wc_bropay_add_gateway( $methods ) {
    	if (!in_array('WC_Gateway_Bropay', $methods)) {
				$methods[] = 'WC_Gateway_Bropay';
			}
			return $methods;
    }


    class WC_Gateway_Bropay extends WC_Payment_Gateway {

	var $ipn_url;

    /**
     * Constructor for the gateway.
     *
     * @access public
     * @return void
     */
	public function __construct() {
		global $woocommerce;

        $this->id           = 'bropay';
        $this->icon         = apply_filters( 'woocommerce_bropay_icon', plugins_url().'/bropay-payment-gateway-for-woocommerce/assets/images/icons/icon.jpg');
        $this->has_fields   = false;
        $this->method_title = __( 'Cryptocoin Payments - ETH, BTC, DOGE...', 'woocommerce' );
        $this->ipn_url   = add_query_arg( 'wc-api', 'WC_Gateway_Bropay', home_url( '/' ) );

		// Load the settings.
		$this->init_form_fields();
		$this->init_settings();

		// Define user set variables
		$this->title 			= $this->get_option( 'title' );
		$this->merchant_id 			= $this->get_option( 'merchant_id' );
        $this->gateway 			= $this->get_option( 'gateway' );
		$this->debug_email			= $this->get_option( 'debug_email' );
		$this->form_submission_method = $this->get_option( 'form_submission_method' ) == 'yes' ? true : false;

		// Logs
		$this->log = new WC_Logger();

		// Actions
		add_action( 'woocommerce_receipt_bropay', array( $this, 'receipt_page' ) );
		add_action( 'woocommerce_update_options_payment_gateways_' . $this->id, array( $this, 'process_admin_options' ) );

		// Payment listener/API hook
		add_action( 'woocommerce_api_wc_gateway_bropay', array( $this, 'check_ipn_response' ) );

		if ( !$this->is_valid_for_use() ) $this->enabled = false;

    }


    /**
     * Check if this gateway is enabled and available in the user's country
     *
     * @access public
     * @return bool
     */
    function is_valid_for_use() {
        return true;
    }

	/**
	 * Admin Panel Options
	 * - Options for bits like 'title' and availability on a country-by-country basis
	 *
	 * @since 1.0.0
	 */
	public function admin_options() {

		?>
		<h3><?php _e( 'Cryptocoin Payments', 'woocommerce' ); ?></h3>
		<p><?php _e( 'Completes checkout via Cryptocoin Payments', 'woocommerce' ); ?></p>

    	<?php if ( $this->is_valid_for_use() ) : ?>

			<table class="form-table">
			<?php
    			// Generate the HTML For the settings form.
    			$this->generate_settings_html();
			?>
			</table><!--/.form-table-->

		<?php else : ?>
            <div class="inline error"><p><strong><?php _e( 'Gateway Disabled', 'woocommerce' ); ?></strong>: <?php _e( 'Cryptocoin Payments does not support your store currency.', 'woocommerce' ); ?></p></div>
		<?php
			endif;
	}


    /**
     * Initialise Gateway Settings Form Fields
     *
     * @access public
     * @return void
     */
    function init_form_fields() {

    	$this->form_fields = array(
			'enabled' => array(
							'title' => __( 'Enable/Disable', 'woocommerce' ),
							'type' => 'checkbox',
							'label' => __( 'Enable Cryptocoin Payments', 'woocommerce' ),
							'default' => 'yes'
						),
			'title' => array(
							'title' => __( 'Title', 'woocommerce' ),
							'type' => 'text',
							'description' => __( 'This controls the title which the user sees during checkout.', 'woocommerce' ),
							'default' => __( 'ETH, BTC, DOGE and other cryptocurrencies', 'woocommerce' ),
							'desc_tip'      => true,
						),
			'merchant_id' => array(
							'title' => __( 'Merchant ID', 'woocommerce' ),
							'type' 			=> 'text',
							'description' => __( 'Please enter your Merchant ID.', 'woocommerce' ),
							'default' => '',
						),
            'gateway' => array(
                'title' => __( 'Gateway', 'woocommerce' ),
                'type' 			=> 'text',
                'description' => __( 'Please enter your Gateway URL.', 'woocommerce' ),
                'default' => '',
            ),
			'debug_email' => array(
							'title' => __( 'Debug Email', 'woocommerce' ),
							'type' => 'email',
							'default' => '',
							'description' => __( 'Send copies of invalid IPNs to this email address.', 'woocommerce' ),
						)
			);

    }


	/**
	 * Get Bropay.io Args
	 *
	 * @access public
	 * @param mixed $order
	 * @return array
	 */
	function get_bropay_args( $order ){

		global $woocommerce;

		$order_id = $order->get_id();

		if ( in_array( $order->get_billing_country(), array( 'US','CA' ) ) ) {
			$order->set_billing_phone(str_replace( array( '( ', '-', ' ', ' )', '.' ), '', $order->get_billing_phone() ));
		}

        foreach($order->get_items() as $item){
            $product = $item->get_product();
            $active_price   = $product->get_price();
            $product_name   = $product->get_name();
            $item_quantity  = $item->get_quantity();
            $sku = $product->get_sku();
            $cart_items_array[] = ["name" => $product_name, "price" => $active_price, "quantity" => $item_quantity, "sku" => $sku];
        }

		// Bropay.io Args
		$bropay_args = array(
                'gateway'              => $this->get_option('gateway'),
				'cmd' 					=> '_pay_auto',
				'merchant' 				=> $this->merchant_id,
				'allow_extra' 				=> 0,
				// Get the currency from the order, not the active currency
				'currency' 		=> $order->get_currency(),
				'reset' 				=> 1,
				'success_url' 				=> $this->get_return_url( $order ),
				'cancel_url'			=> esc_url_raw($order->get_cancel_order_url_raw()),

				// Order key + ID
				'invoice'				=> $this->invoice_prefix . $order->get_order_number(),
				'custom' 				=> serialize( array( $order->get_id(), $order->get_order_key() ) ),

				// IPN
				'ipn_url'			=> $this->ipn_url,

				// Billing Address info
				'first_name'			=> $order->get_billing_first_name(),
				'last_name'				=> $order->get_billing_last_name(),
				'email'					=> $order->get_billing_email(),
                'items'              => serialize($cart_items_array)
		);



			$bropay_args = array_merge($bropay_args, array(
				'want_shipping'         =>  1,
				'company'				=> $order->get_billing_company(),
				'address1'				=> $order->get_billing_address_1(),
				'address2'				=> $order->get_billing_address_2(),
				'city'					=> $order->get_billing_city(),
				'state'					=> $order->get_billing_state(),
				'zip'					=> $order->get_billing_postcode(),
				'country'				=> $order->get_billing_country(),
				'phone'					=> $order->get_billing_phone(),
			));

		if ($this->simple_total) {
			$bropay_args['item_name'] 	= sprintf( __( 'Order %s' , 'woocommerce'), $order->get_order_number() );
			$bropay_args['quantity'] 		= 1;
			$bropay_args['amountf'] 		= number_format( $order->get_total(), 8, '.', '' );
			$bropay_args['taxf'] 				= 0.00;
			$bropay_args['shippingf']		= 0.00;
		} else if ( wc_tax_enabled() && wc_prices_include_tax() ) {
			$bropay_args['item_name'] 	= sprintf( __( 'Order %s' , 'woocommerce'), $order->get_order_number() );
			$bropay_args['quantity'] 		= 1;
			$bropay_args['amountf'] 		= number_format( $order->get_total() - $order->get_total_shipping() - $order->get_shipping_tax(), 8, '.', '' );
			$bropay_args['shippingf']		= number_format( $order->get_total_shipping() + $order->get_shipping_tax() , 8, '.', '' );
			$bropay_args['taxf'] 				= 0.00;
		} else {
			$bropay_args['item_name'] 	= sprintf( __( 'Order %s' , 'woocommerce'), $order->get_order_number() );
			$bropay_args['quantity'] 		= 1;
			$bropay_args['amountf'] 		= number_format( $order->get_total() - $order->get_total_shipping() - $order->get_total_tax(), 8, '.', '' );
			$bropay_args['shippingf']		= number_format( $order->get_total_shipping(), 8, '.', '' );
			$bropay_args['taxf']				= number_format( $order->get_total_tax(), 8, '.', '' );
		}

		$bropay_args = apply_filters( 'woocommerce_bropay_args', $bropay_args );

		return $bropay_args;
	}

    /**
	 * Generate the bropay button link
     *
     * @access public
     * @param mixed $order_id
     * @return string
     */
    function generate_bropay_url($order) {

		global $woocommerce;

		if ( $order->get_status() != 'completed' && get_post_meta($order->get_id(), 'Bropay payment complete', true ) != 'Yes' ) {
			$order->update_status('pending', 'Customer is being redirected to Cryptocoin Payments...');
		}

		$bropay_adr = plugin_dir_url( __FILE__ ).'transfer.php?';
		$bropay_args = $this->get_bropay_args( $order );
		$bropay_adr .= http_build_query( $bropay_args, '', '&' );
		return $bropay_adr;
	}

    /**
     * Process the payment and return the result
     *
     * @access public
     * @param int $order_id
     * @return array
     */
	function process_payment( $order_id ){

		$order = wc_get_order( $order_id );

		return array(
				'result' 	=> 'success',
				'redirect'	=> $this->generate_bropay_url($order),
		);
	}

    /**
     * Output for the order received page.
     *
     * @access public
     * @return void
     */
	function receipt_page( $order ) {
		echo '<p>'.__( 'Thank you for your order, please click the button below to pay with Cryptocoin Payments.', 'woocommerce' ).'</p>';

		echo $this->generate_bropay_form( $order );
	}

    private function _errorAndDie($error_msg)
    {
            $email = $this->get_option('debug_email');
            if (!empty($email)) {
                $report = "Error Message: " . $error_msg . "\n\n";

                $report .= "AUTH User: " . $_SERVER['PHP_AUTH_USER'] . "\n";
                $report .= "AUTH Pass: " . $_SERVER['PHP_AUTH_PW'] . "\n\n";

                $report .= "POST Fields\n\n";
                foreach ($_POST as $key => $value) {
                    $report .= $key . '=' . html_entity_decode($value, ENT_QUOTES, 'UTF-8') . "\n";
                }

                wp_mail($email, "Coin Payments Invalid IPN", $report);
            }
            die('IPN Error: ' . $error_msg);
    }

	/**
	 * Check bropay.io IPN validity
	 **/
	function check_ipn_request_is_valid($ipn){

        if (!isset($ipn['ipn_mode'])) {
            $this->_errorAndDie('IPN received with no ipn_mode.');
        }
        if ($ipn['ipn_mode'] == 'hmac') {
            if (!isset($_SERVER['HTTP_HMAC']) || empty($_SERVER['HTTP_HMAC'])) {
                $this->_errorAndDie('No HMAC signature sent.');
            }

            $request = file_get_contents('php://input');
            if ($request === FALSE || empty($request)) {
                $this->_errorAndDie('Error reading POST data: ' . print_r($_SERVER, TRUE) . '/' . print_r($_POST, TRUE));
            }

            $merchant = isset($ipn['merchant']) ? $ipn['merchant'] : '';
            if (empty($merchant)) {
                $this->_errorAndDie('No Merchant ID passed');
            }
            if ($merchant != trim($this->get_option('merchant_id'))) {
                $this->_errorAndDie('Invalid Merchant ID');
            }

            return TRUE;
        } else if ($ipn['ipn_mode'] == 'httpauth') {
            $merchant = isset($ipn['merchant']) ? $ipn['merchant'] : '';
            if (empty($merchant)) {
                $this->_errorAndDie('No Merchant ID passed');
            }
            if ($merchant != trim($this->get_option('merchant_id'))) {
                $this->_errorAndDie('Invalid Merchant ID');
            }
            return TRUE;
        } else {
            $this->_errorAndDie('Unknown ipn_mode.');
        }
        return FALSE;
	}

	/**
	 * Successful Payment!
	 *
	 * @access public
	 * @param array $posted
	 * @return void
	 */
	function successful_request( $posted ) {
        $order_id = intval($posted['invoice']);
        $order = wc_get_order( $order_id );
        if ($posted['ipn_type'] == 'button') {
            if ($posted['currency'] == $order->get_currency()){
                if ($posted['amount'] >= $order->get_subtotal()) {
                    $status = intval($posted['status']);
                    if ($status < 0) {
                        //canceled or timed out
                        $order->update_status('canceled', 'order has been canceled');
                    } else if ($status >= 100 || $status == 2) {
                        //order complete or queued for nightly payout
                        $str = 'Payment Status: ' . $posted['status_text'] . '<br />';
                        $str .= 'Transaction ID: ' . $posted['txn_id'] . '<br />';
                        $str .= 'Original Amount: ' . sprintf('%.08f', $posted['amount']) . '<br />';
                        $str .= 'Received Amount: ' . sprintf('%.08f', $posted['amount']);
                        $order->update_status('processing');
                    } else {
                        //order pending
                        $order->update_status('pending');
                    }
                    $order->save();
                    die('IPN OK');
                } else {
                    $this->_errorAndDie('Amount paid is less than order total!', $order);
                }
            } else {
                $this->_errorAndDie('Original currency does not match!', $order);
            }
        } else {
            $this->_errorAndDie('Invalid IPN type!', $order);
        }
	}

	/**
	 * Check for bropay IPN Response
	 *
	 * @access public
	 * @return void
	 */
	function check_ipn_response() {

		@ob_clean();

		if ( ! empty( $_POST ) && $this->check_ipn_request_is_valid($_POST) ) {
			$this->successful_request($_POST);
		} else {
			$this->_errorAndDie("request is empty or not correct");
 		}
	}

	/**
	 * get_bropay_order function.
	 *
	 * @access public
	 * @param mixed $posted
	 * @return void
	 */
	function get_bropay_order( $posted ) {
		$custom = maybe_unserialize( stripslashes_deep($posted['custom']) );

    	// Backwards comp for IPN requests
    	if ( is_numeric( $custom ) ) {
	    	$order_id = (int) $custom;
	    	$order_key = $posted['invoice'];
    	} elseif( is_string( $custom ) ) {
	    	$order_id = (int) str_replace( $this->invoice_prefix, '', $custom );
	    	$order_key = $custom;
    	} else {
    		list( $order_id, $order_key ) = $custom;
		}

		$order = wc_get_order( $order_id );

		if ($order === FALSE) {
			// We have an invalid $order_id, probably because invoice_prefix has changed
			$order_id 	= wc_get_order_id_by_order_key( $order_key );
			$order 		= wc_get_order( $order_id );
		}

		// Validate key
		if ($order === FALSE || $order->get_order_key() !== $order_key ) {
			return FALSE;
		}

		return $order;
	}

}

class WC_Bropay extends WC_Gateway_Bropay {
	public function __construct() {
		_deprecated_function( 'WC_Bropay', '1.4', 'WC_Gateway_Bropay' );
		parent::__construct();
	}
  }
}